#pragma once

#include <Windows.h>

class SectionLock {
public:
    explicit SectionLock() noexcept;
    explicit SectionLock(const SectionLock& ) = delete;
    SectionLock& operator= (const SectionLock&) = delete;
    ~SectionLock();

    void Lock();
    bool TryLock();
    void UnLock();

private:
    CRITICAL_SECTION section_;
};

template <class Lock>
class Locker {
public:
    explicit Locker(Lock& lock) noexcept : lock_(lock) {
        lock_.Lock();
    }
    explicit Locker(const Locker&) = delete;
    Locker& operator= (const Locker&) = delete;
    ~Locker() {
        lock_.UnLock();
    }

private:
    Lock& lock_;
};

